# -*- Perl -*-
# Supplemental classes for xdbtoltx
#
##############################################################################

if ($VERBOSE) {
    local($VERSION) = '$Id: xdbtoltx.cls 1.1 1995/03/06 13:07:47 norm Exp norm $'; # '
    local($REQNAME) = (split(/\s+/, $VERSION))[1];
    local($vers) = (split(/\s+/, $VERSION))[2];

    print STDERR "Supplemental $REQNAME version $vers.\n";
}

##############################################################################

{
    package BOOK;
    @ISA = ('GENERIC_TAG');

    sub new {
        return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($preamble, $postamble);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$preamble = &main::cfg($main::bookfiles, 
			       "$main::PROGNAME filter", 
			       'preamble');
	$postamble = &main::cfg($main::bookfiles, 
				"$main::PROGNAME filter", 
				'postamble');

	if ($preamble) {
	    open (F, "$preamble");
	    while (<F>) {
		$markup->markup($_);
	    }
	    close (F);
	}
	$markup->markup($self->markup_start());

	$self->markup_children($markup, $self->content());

	if ($postamble) {
	    open (F, "$postamble");
	    while (<F>) {
		$markup->markup($_);
	    }
	    close (F);
	}
	$markup->markup($self->markup_end());
	
	$inmarkup ? $markup : $markup->output();
    }
}

{
    package BOOKINFO;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($startdoc) = 0;
	my($child);

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup($markup);

	$markup->markup($self->markup_start());

	$child = $self->content();
	while (!$self->null($child)) {
	    if ($child->tag('BOOKBIBLIO')) {
		$child->markup($markup);
		$markup->markup("\\begin{document}\n");
		$markup->markup("\\maketitle\n");
		$startdoc = 1;
	    } else {
		$child->markup($markup);
	    }
	    $child = $child->next();
	}

	if (!$startdoc) {
	    $markup->markup("\\begin{document}\n");
	}

	$markup->markup($self->markup_end());

	$markup->markup("\\newpage\n");
	$markup->markup("\\tableofcontents\n");
	$markup->markup("\\newpage\n");
	
	$inmarkup ? $markup : $markup->output();
    }
}

{
    package BOOKBIBLIO;
    @ISA = ('GENERIC_TITLED');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($child) = $self->content();
	my($titlemarkup) = new MARKUP_OBJECT;

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup($markup);

	$self->markup_title($titlemarkup);
	
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($child) = $self->content();
	my($title) = "";

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup($markup);

	if ($child->{'TAG'} eq 'TITLE') {
	    $title = $child->markup();
	    $title =~ s/<\001lb>/ /g;
	    $child = $child->next();
	}

	if ($child->{'TAG'} eq 'SUBTITLE') {
	    $title .= "\\\\" . $child->markup();
	    $title =~ s/<\001lb>/ /g;
	    $child = $child->next();
	}

	$markup->markup("\\title\{$title\}\n");
	$self->markup_children($markup, $child);
	$markup->markup($self->markup_end());

	$inmarkup ? $markup : $markup->output();
    }
}

{
    package AUTHORGROUP;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($first);
	local($_);

	@authstack = ();

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup($markup);

	$self->markup_children($markup, $self->content());

	$_ = shift @authstack;

	if (@authstack) {
	    $markup->markup("\\author\{$_\%\n");
	    $first = 1;
	    $markup->markup("\\thanks{%\n");
	    while (@authstack) {
		$_ = shift @authstack;
		if (@authstack) {
		    $markup->text(", \%\n") if !$first;
		    $markup->text("$_");
		} else {
		    $markup->text(", \%\n") if !$first;
		    $markup->text("and $_.\}}\n");
		}
		$first = 0;
	    }
	} else {
	    $markup->markup("\\author\{$_}\n");
	}
	
	$inmarkup ? $markup : $markup->output();
    }
}

{
    package CORPAUTHOR;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	push (@AUTHORGROUP::authstack, $name);

	$inmarkup ? $markup : $name;
    }
}

{
    package AUTHOR;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$firstname = "";
	$surname = "";
	$jobtitle = "";

	$self->markup_children($markup, $self->content());

	$_ = "$firstname $surname";
	$_ .= " ($jobtitle)" if $jobtitle ne "";

	push (@AUTHORGROUP::authstack, $_);

	$inmarkup ? $markup : $_;
    }
}

{
    package FIRSTNAME;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	$AUTHOR::firstname = $name;

	$inmarkup ? $markup : $name;
    }
}

{
    package SURNAME;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	$AUTHOR::surname = $name;

	$inmarkup ? $markup : $name;
    }
}

{
    package AFFILIATION;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$jobtitle = "";

	$self->markup_children($markup, $self->content());

	$AUTHOR::jobtitle = $jobtitle;

	$inmarkup ? $markup : $jobtitle
    }

}

{
    package JOBTITLE;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	$AFFILIATION::jobtitle = $name;

	$inmarkup ? $markup : $name;
    }
}

{
    package PROGRAMLISTING;
    @ISA = ('GENERIC_OBEYSPACES');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup($markup);

	$markup->{'OBEY_SPACES'}++;
	
	if ($self->role('TDSSUMMARY')) {
	    $markup->markup("\\begin{tdsSummary}");
	    $self->markup_children($markup, $self->content());
	    $markup->markup("\\end{tdsSummary}");
	} else {
	    $markup->markup($self->markup_start());
	    $self->markup_children($markup, $self->content());
	    $markup->markup($self->markup_end());
	}

	$markup->{'OBEY_SPACES'}--;

	$inmarkup ? $markup : $markup->output();
    }
}

{
    package ACRONYM;
    @ISA = ('GENERIC_INLINE');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($text);

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup();

	$markup->markup($self->markup_start());
	$text = $self->markup_children(undef, $self->content());
	$text =~ tr/A-Z/a-z/;
	$markup->text($text);
	$markup->markup($self->markup_end());

	$inmarkup ? $markup : $markup->output();
    }
}

1;

