# -*- Perl -*-
# Supplemental classes for xdbtotexi
#
##############################################################################

if ($VERBOSE) {
    local($VERSION) = '$Id: xdbtotexi.cls 1.1 1995/03/06 13:07:47 norm Exp norm $'; # '
    local($REQNAME) = (split(/\s+/, $VERSION))[1];
    local($vers) = (split(/\s+/, $VERSION))[2];

    print STDERR "Supplemental $REQNAME version $vers.\n";
}

##############################################################################

{
    package BOOK;
    @ISA = ('GENERIC_TAG');

    sub new {
        return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($preamble, $postamble);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$preamble = &main::cfg($main::bookfiles, 
			       "$main::PROGNAME filter",
			       'preamble');
	$postamble = &main::cfg($main::bookfiles, 
				"$main::PROGNAME filter", 
				'postamble');

	if ($preamble) {
	    open (F, "$preamble");
	    while (<F>) {
		$markup->markup($_);
	    }
	    close (F);
	}

	$markup->markup($self->markup_start());

	$self->markup_children($markup, $self->content());

	if ($postamble) {
	    open (F, "$postamble");
	    while (<F>) {
		$markup->markup($_);
	    }
	    close (F);
	}

	$markup->markup("\@contents");

	$markup->markup($self->markup_end());
	
	$inmarkup ? $markup : $markup->output();
    }
}

{
    package BOOKINFO;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($discard, $next_title, $next, $top);

	$top = &main::cfg($main::bookfiles, $main::FILTERCFG, 'top-node');

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$discard = new MARKUP_OBJECT;

	$markup->markup($self->markup_start());
	$self->markup_children($discard, $self->content());

	$next_title = $self->next_node(('CHAPTER'));
	$DOC_NODE = "\@node Top, $next_title, (dir), (dir)";

	if ($top) {
	    open (F, $top);
	    while (<F>) {
		s/%%TITLE%%/$DOC_TITLE/g;
		s/%%NODE%%/$DOC_NODE/g;
		s/%%SUBTITLE%%/$DOC_SUBTITLE/g;
		s/%%AUTHOR%%/$DOC_AUTHOR/g;
		s/%%THANKS%%/$DOC_THANKS/g;
		s/%%LEGALNOTICE%%/$DOC_LEGALNOTICE/g;
		$markup->raw_markup($_);
	    }
	    close (F);
	}

	$markup->markup("\@ifinfo");

	$markup->markup("\@menu");
	$next = $self->content();
	while ($next) {
	    if ($next->tag('CHAPTER') || $next->tag('APPENDIX')) {
		$next_title = ($next->content())->markup();
		$next_title =~ s/<\001lb>//g;
		$markup->markup("* ${next_title}::\n");
	    }
	    $next = $next->next_element();
	}
	$markup->markup("\@end menu");

	$markup->markup("\@end ifinfo");

	$markup->markup($self->markup_end());

	$inmarkup ? $markup : $markup->output();
    }
}

{
    package BOOKBIBLIO;
    @ISA = ('GENERIC_TITLED');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($child) = $self->content();
	my($title) = "";

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup();

	if ($child->{'TAG'} eq 'TITLE') {
	    $title = $child->markup();
	    $title =~ s/<\001lb>/ /g;
	    $child = $child->next();
	}

	$BOOKINFO::DOC_TITLE = $title;
	$markup->markup("\@title $title");

	if ($child->{'TAG'} eq 'SUBTITLE') {
	    $title = $child->markup();
	    $title =~ s/<\001lb>/ /g;

	    $BOOKINFO::DOC_SUBTITLE = $title;
	    $markup->markup("\@subtitle $title");

	    $child = $child->next();
	}

	$self->markup_children($markup, $child);
	$markup->markup($self->markup_end());

	$inmarkup ? $markup : $markup->output();
    }
}

{
    package AUTHORGROUP;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($first);
	local($_);

	@authstack = ();

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup();

	$self->markup_children($markup, $self->content());

	$_ = shift @authstack;

	$BOOKINFO::DOC_AUTHOR = "";
	$BOOKINFO::DOC_THANKS = "";

	if (@authstack) {
	    $BOOKINFO::DOC_AUTHOR = $_;
	    $markup->markup("\@author $_");
	    $first = 1;
	    while (@authstack) {
		$_ = shift @authstack;
		if (@authstack) {
		    if (!$first) {
			$markup->text(",\n");
			$BOOKINFO::DOC_THANKS .= ", ";
		    }
		    $BOOKINFO::DOC_THANKS .= $_;
		    $markup->text($_);
		} else {
		    if (!$first) {
			$markup->text(",\n");
			$BOOKINFO::DOC_THANKS .= ", ";
		    }
		    $BOOKINFO::DOC_THANKS .= "and $_.";
		    $markup->text("and $_.");
		}
		$first = 0;
	    }
	} else {
	    $BOOKINFO::DOC_AUTHOR = $_;
	    $markup->markup("\@author $_");
	}
	
	$inmarkup ? $markup : $markup->output();
    }
}

{
    package CORPAUTHOR;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	push (@AUTHORGROUP::authstack, $name);

	$inmarkup ? $markup : $name;
    }
}

{
    package AUTHOR;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$firstname = "";
	$surname = "";
	$jobtitle = "";

	$self->markup_children($markup, $self->content());

	$_ = "$firstname $surname";
	$_ .= " ($jobtitle)" if $jobtitle ne "";

	push (@AUTHORGROUP::authstack, $_);

	$inmarkup ? $markup : $_;
    }
}

{
    package FIRSTNAME;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	$AUTHOR::firstname = $name;

	$inmarkup ? $markup : $name;
    }
}

{
    package SURNAME;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	$AUTHOR::surname = $name;

	$inmarkup ? $markup : $name;
    }
}

{
    package AFFILIATION;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$jobtitle = "";

	$self->markup_children($markup, $self->content());

	$AUTHOR::jobtitle = $jobtitle;

	$inmarkup ? $markup : $jobtitle
    }

}

{
    package JOBTITLE;
    @ISA = ('GENERIC_TAG');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($namemarkup, $name);
	local($_);

	$markup = new MARKUP_OBJECT if !$inmarkup;

	$namemarkup = new MARKUP_OBJECT;
	$self->markup_children($namemarkup, $self->content());
	$name = $namemarkup->output();

	$AFFILIATION::jobtitle = $name;

	$inmarkup ? $markup : $name;
    }
}

{
    package LEGALNOTICE;
    @ISA = ('GENERIC_WRAPPER');

    sub new {
	return bless { };
    }

    sub markup {
	my($self, $markup) = @_;
	my($inmarkup) = defined($markup);
	my($noticemarkup);

	$markup = new MARKUP_OBJECT if !$inmarkup;
	$self->default_markup($markup);

	$noticemarkup = new MARKUP_OBJECT;

	$self->markup_children($noticemarkup, $self->content());

	$BOOKINFO::DOC_LEGALNOTICE = $noticemarkup->output();

	$inmarkup ? $markup : $markup->output();
    }
}


1;





